import java.awt.geom.Arc2D;
import java.awt.geom.Arc2D.Double;

/**
 * Zeichnet einen Kreissektor auf die Leinwand.
 * 
 * @author Bernd Gramlich nach einer Vorlage von Michael Klling und David J. Barnes 
 * @version 2013-10-14
 */
public class Kreissektor
{
    // Attribute
    private int positionX;
    private int positionY;
    private int radius;
    private int anfangswinkel;
    private int gesamtwinkel;
    private String farbe;
    private boolean sichtbar;

    /**
     * Konstruktor der Klasse Kreissektor.
     * Erzeugt einen neuen Kreissektor in Standardgre und Standardfarbe.
     */
    public Kreissektor ()
    {
        radius        = 50;
        anfangswinkel = 30;
        gesamtwinkel  = 300;
        farbe         = "gelb";

        werdeSichtbar ();
    }

    /**
     * Mache diesen Kreissektor sichtbar.
     */
    public void werdeSichtbar ()
    {
        sichtbar = true;
        zeichne ();
    }

    /**
     * Mache diesen Kreissektor unsichtbar.
     */
    public void werdeUnsichtbar ()
    {
        loesche ();
        sichtbar = false;
    }

    /**
     * ndere den Radius dieses Kreissektors.
     */
    public void setzeRadius (int neuerRadius)
    {
        radius = neuerRadius;
        zeichne ();
    }

    /**
     * ndere den Anfangswinkel dieses Kreissektors.
     */
    public void setzeAnfangswinkel (int neuerAnfangswinkel)
    {
        anfangswinkel = neuerAnfangswinkel;
        zeichne ();
    }

    /**
     * ndere den Gesamtwinkel dieses Kreissektors.
     */
    public void setzeGesamtwinkel (int neuerGesamtwinkel)
    {
        gesamtwinkel = neuerGesamtwinkel;
        zeichne ();
    }

    /**
     * ndere die Farbe dieses Kreissektors in 'neueFarbe'. Gltige Angaben
     * sind in der Methode setzeZeichenfarbe () der Klasse Leinwand
     * definiert. (Zuletzt waren das "blau", "gelb", "grau", "gruen", "lila",
     * "orange", "rot", "schwarz", "tuerkis" und "weiss".)
     */
    public void setzeFarbe (String neueFarbe)
    {
        farbe = neueFarbe;
        zeichne ();
    }

    /**
     * ndere die Koordinaten dieses Kreissektors.
     */
    public void setzePosition (int neuePositionX, int neuePositionY)
    {
        positionX = neuePositionX;
        positionY = neuePositionY;
        zeichne ();
    }

    /**
     * Verschiebe diesen Kreissektor waagrecht um 'entfernung'
     * Bildschirmpunkte.
     */
    public void verschiebeWaagrecht (int entfernung)
    {
        loesche ();
        positionX += entfernung;
        zeichne ();
    }

    /**
     * Verschiebe diesen Kreissektor senkrecht um 'entfernung'
     * Bildschirmpunkte.
     */
    public void verschiebeSenkrecht (int entfernung)
    {
        loesche ();
        positionY += entfernung;
        zeichne ();
    }

    /**
     * Gleite diesen Kreissektor waagrecht um 'entfernung' Bildschirmpunkte.
     */
    public void gleiteWaagrecht (int entfernung)
    {
        int delta;

        if (entfernung < 0)
        {
            delta = -1;
            entfernung = -entfernung;
        }
        else
        {
            delta = 1;
        }

        for (int i = 0; i < entfernung; i++)
        {
            positionX += delta;
            zeichne ();
        }
    }

    /**
     * Gleite diesen Kreissektor senkrecht um 'entfernung' Bildschirmpunkte.
     */
    public void gleiteSenkrecht (int entfernung)
    {
        int delta;

        if (entfernung < 0)
        {
            delta = -1;
            entfernung = -entfernung;
        }
        else
        {
            delta = 1;
        }

        for (int i = 0; i < entfernung; i++)
        {
            positionY += delta;
            zeichne ();
        }
    }

    /**
     * Zeichne diesen Kreissektor auf die Leinwand.
     */
    private void zeichne ()
    {
        if (sichtbar)
        {
            Leinwand leinwand = Leinwand.gibLeinwand ();
            leinwand.zeichne (this, farbe, new Arc2D.Double (positionX,
                    positionY, 2*radius, 2*radius, anfangswinkel, gesamtwinkel,
                    2));
        }
    }

    /**
     * Lsche diesen Kreissektor von der Leinwand.
     */
    private void loesche ()
    {
        if (sichtbar)
        {
            Leinwand leinwand = Leinwand.gibLeinwand ();
            leinwand.entferne (this);
        }
    }
}
